import json
import os.path
from typing import TypeVar

from pydantic import BaseModel

config_file = 'data/config.json'

if not os.path.exists('data'):
    os.mkdir('data')


class BotSettings(BaseModel):
    token: str = ''
    owners: list[str] = []
    mongo: str = ''
    debug: bool = True
    database: str = ''


class BasicBotConfig(BaseModel):
    bot: BotSettings = BotSettings()

    def __init__(self):
        if os.path.exists(config_file):
            with open(config_file, 'r', encoding='utf-8') as f:
                super().__init__(**json.load(f))
        else:
            super().__init__()

        with open(config_file, 'w', encoding='utf-8') as f:
            json.dump(self.model_dump(), f, ensure_ascii=True, sort_keys=True, indent=4)


R = TypeVar('R')



from typing import Type

from beanie import init_beanie
from motor.motor_asyncio import AsyncIOMotorClient

from .types import *


async def connect(mongo: str, database: str, models: list[Type[Document]]):
    client = AsyncIOMotorClient(mongo)

    await init_beanie(database=client[database],
                      document_models=models)



import asyncio
from datetime import datetime
from typing import Union

from aiogram import Bot
from aiogram.exceptions import AiogramError
from aiogram.types import Message, CallbackQuery
from aiogram.utils.keyboard import InlineKeyboardBuilder


def get_timestamp(time: int) -> str:
    return datetime.fromtimestamp(time + 3600 * 3).strftime("%H:%M %d.%m.%Y")


async def notify(bot: Bot, user_id: int, text: str):
    try:
        await bot.send_message(user_id, text)
    except AiogramError:
        pass


async def distribute(bot: Bot, ids: list[int], text: str, additional_tasks: list[...] | None = None) -> tuple[...]:
    if additional_tasks is None:
        additional_tasks = []

    tasks = additional_tasks
    for user_id in ids:
        tasks.append(notify(bot, user_id, text))
    return await asyncio.gather(*tasks)


async def confirm_action(data: Union[Message, CallbackQuery], description: str, warning: bool,
                         callback_data: str):
    builder = InlineKeyboardBuilder()

    builder.button(text='✅ Подтвердить', callback_data=callback_data)
    builder.button(text='❌ Отмена', callback_data='state_clear')

    if isinstance(data, Message):
        # noinspection PyTypeChecker
        func = data.answer
    else:
        # noinspection PyTypeChecker
        func = data.message.edit_text

    await func(
        f'Вы уверены, что хотите {description}?' + ('\n\n⚠️ Это действие необратимо' if warning else ''),
        reply_markup=builder.as_markup())



import asyncio
import random
import string

from aiogram import Dispatcher, Router, Bot
from aiogram.client.session.aiohttp import AiohttpSession
from aiogram.enums import ParseMode
from aiogram.exceptions import TelegramUnauthorizedError
from aiogram.types import User
from aiogram.webhook.aiohttp_server import SimpleRequestHandler, setup_application, TokenBasedRequestHandler
from aiohttp import web


class BaseBot:
    def __init__(self, token: str):
        self.token = token

        self.router = Router()
        self.dispatcher = Dispatcher()
        self.dispatcher.include_router(self.router)

        self._bot_settings = {"session": AiohttpSession(), "parse_mode": ParseMode.HTML}
        self.main_bot = Bot(self.token, **self._bot_settings)

    async def start(self) -> None:
        pass


class XBot(BaseBot):
    def __init__(self, token: str):
        super().__init__(token)

    async def start(self) -> None:
        await self.dispatcher.start_polling(self.main_bot)


MAIN_BOT_PATH = "/webhook/main"
OTHER_BOTS_PATH = "/webhook/bot/{bot_token}"


class XMultiBot(BaseBot):
    def __init__(self, main_token: str, base_url: str, port: int):
        super().__init__(main_token)
        self.minions: dict[str, Bot] = {}

        self._secret = ''.join([random.choice(string.ascii_letters + string.digits) for _ in range(64)])
        self._port = port
        self._base_url = base_url
        self._startup_tokens = []

        self.app = web.Application()
        SimpleRequestHandler(dispatcher=self.dispatcher, bot=self.main_bot,
                             secret_token=self._secret).register(self.app,
                                                                 path=MAIN_BOT_PATH)
        TokenBasedRequestHandler(
            dispatcher=self.dispatcher,
            bot_settings=self._bot_settings
        ).register(self.app, path=OTHER_BOTS_PATH)

        setup_application(self.app, self.dispatcher, bot=self.main_bot)

        async def on_startup(bot: Bot):
            for token in self._startup_tokens:
                new_bot = Bot(token, self.main_bot.session)
                try:
                    await new_bot.get_me()
                except TelegramUnauthorizedError:
                    continue
                await new_bot.delete_webhook(drop_pending_updates=True)
                await new_bot.set_webhook(f'{self._base_url}/webhook/bot/{token}')
                self.minions[token] = new_bot

            await bot.set_webhook(f"{self._base_url}{MAIN_BOT_PATH}", secret_token=self._secret)

        self.dispatcher.startup.register(on_startup)

    async def add_minion(self, token: str) -> User | None:
        new_bot = Bot(token, **self._bot_settings)
        try:
            bot_user = await new_bot.get_me()
        except TelegramUnauthorizedError:
            return None
        await new_bot.delete_webhook(drop_pending_updates=True)
        await new_bot.set_webhook(f'{self._base_url}{OTHER_BOTS_PATH.format(bot_token=token)}')
        self.minions[token] = new_bot

        return bot_user

    async def delete_minion(self, token: str) -> bool:
        if token in self.minions:
            await self.minions[token].delete_webhook()
            del self.minions[token]
            return True
        else:
            return False

    def register_minions(self, tokens: list[str]):
        self._startup_tokens = tokens

    async def start(self):
        runner = web.AppRunner(self.app)
        await runner.setup()

        site = web.TCPSite(runner, '0.0.0.0', self._port)
        await site.start()
        await asyncio.Event().wait()



import math
import time
import traceback

from aiogram import Router, F, Bot
from aiogram.exceptions import AiogramError
from aiogram.types import Message, ErrorEvent, BufferedInputFile, CallbackQuery
from aiogram.utils.keyboard import InlineKeyboardBuilder

message_limit = 3970


def get_support_markup(url: str):
    builder = InlineKeyboardBuilder()

    builder.button(text='🆘 Поддержка', url=url)
    builder.adjust(1)

    return builder.as_markup()


def get_user_markup(user_url: str):
    builder = InlineKeyboardBuilder()

    builder.button(text='👤 Пользователь', url=user_url)
    builder.adjust(1)

    return builder.as_markup()


async def report(bot: Bot, owners: list[int], object_name: str, text: str, username: str,
                 user_id: int,
                 user_url: str):
    traceback_log = BufferedInputFile(traceback.format_exc().encode(),
                                      filename=f"Traceback{math.floor(time.time())}.txt")
    for admin in owners:
        try:
            await bot.send_document(admin, traceback_log,
                                    caption=f'⚠️ Произошла ошибка при обработке {object_name} от @{username} [<code>{user_id}</code>]!\n'
                                            f'<pre>{text}</pre>',
                                    reply_markup=get_user_markup(user_url))
        except AiogramError:
            pass


def add_to_router(main_router: Router, main_bot: Bot, owners: list[int], url: str):
    error_handler_router = Router()

    @error_handler_router.error(F.update.message.as_("message"))
    async def error_handler(exception: ErrorEvent, message: Message):
        await message.reply('❌ Похоже, что-то пошло не так, репорт отправлен', reply_markup=get_support_markup(url))

        await report(main_bot, owners, 'сообщения', message.text, message.from_user.username,
                     message.from_user.id,
                     message.from_user.url)

    @error_handler_router.error(F.update.callback_query.as_("query"))
    async def error_handler(exception: ErrorEvent, query: CallbackQuery):
        await query.answer('❌ Похоже, что-то пошло не так, репорт отправлен', show_alert=True)

        await report(main_bot, owners, 'кнопки', query.data, query.from_user.username, query.from_user.id,
                     query.from_user.url)

    main_router.include_router(error_handler_router)
